/*
  tictactoe.c
  
  A two-player, CGI-based tic-tac-toe
*/


#include <stdio.h>
#include <stdlib.h>
#include <cgi-util.h>


#define EMPTY 0

char pieces[3] = {'_', 'X', 'O'};

int board[3][3];


/* Local function prototype: */

void ahref(int turn, int x, int y);


/* --- MAIN --- */

int main(int argc, char * argv[])
{
  int turn, x, y, xx, yy, winner;
  const char * tmp;
  
  
  /* Init CGI: */
  
  cgi_init();
  printf("Content-type: text/html\n\n");
  
  
  turn = cgi_getentryint("turn");
  
  if (turn == 0)
    {
      /* We must be starting a new game: */
      
      printf("Welcome to a new game of tic-tac-toe!<p>\n");
      
      
      /* (initialize the board to all blanks) */
      
      for (yy = 0; yy < 3; yy++)
	{
	  for (xx = 0; xx < 3; xx++)
	    {
	      board[yy][xx] = EMPTY;
	    }
	}
      
      
      /* (player 1 goes first) */
      
      turn = 1;
    }
  else
    {
      /* Someone's turn: */
      
      
      /* (load board) */
      
      tmp = cgi_getentrystr("board");
      
      for (yy = 0; yy < 3; yy++)
	{
	  for (xx = 0; xx < 3; xx++)
	    {
	      board[yy][xx] = tmp[yy * 3 + xx] - '0';
	    }
	}
      

      /* (apply their move) */
      
      x = cgi_getentryint("x");
      y = cgi_getentryint("y");
      
      board[y][x] = turn;

      
      /* Toggle who's turn it is: */
      
      if (turn == 1)
	turn = 2;
      else
	turn = 1;
    }
  
  
  /* See if anyone won: */
  
  winner = EMPTY;
  
  /* (horizontal win?) */
  
  for (yy = 0; yy < 3; yy++)
    {
      if (board[yy][0] != EMPTY &&
	  board[yy][0] == board[yy][1] &&
	  board[yy][1] == board[yy][2])
	{
	  winner = board[yy][0];
	}
    }


  /* (vertical win?) */
  
  for (xx = 0; xx < 3; xx++)
    {
      if (board[0][xx] != EMPTY &&
	  board[0][xx] == board[1][xx] &&
	  board[1][xx] == board[2][xx])
	{
	  winner = board[0][xx];
	}
    }
  
  
  /* (diagonal win - top-left to bottom-right?) */
  
  if (board[0][0] != EMPTY &&
      board[0][0] == board[1][1] &&
      board[1][1] == board[2][2])
    {
      winner = board[0][0];
    }


  /* (diagonal win - top-left to bottom-right?) */
  
  if (board[2][0] != EMPTY &&
      board[2][0] == board[1][1] &&
      board[1][1] == board[0][2])
    {
      winner = board[2][0];
    }

  
  if (winner != EMPTY)
    {
      printf("Congratulations!  %c won!<p>\n", pieces[winner]);
    }
  else
    {
      printf("It is %c's turn<p>\n", pieces[turn]);
    }
  
  
  /* Display the playing board: */
  
  for (yy = 0; yy < 3; yy++)
    {
      for (xx = 0; xx < 3; xx++)
	{
	  if (board[yy][xx] == 0 && winner == EMPTY)
	    ahref(turn, xx, yy);
	  
	  printf("<img src=\"%c.gif\" alt=\"%c\" width=32 height=32 border=0>",
		 pieces[board[yy][xx]],
		 pieces[board[yy][xx]]);
	  
	  if (board[yy][xx] == 0 && winner == EMPTY)
	    printf("</a>");
	}
      
      printf("<br>\n");
    }  
  
  
  /* Restart link: */
  
  printf("<p><a href=\"tictactoe.cgi\">Start over</a><p>\n");
  
  
  /* Close up and quit: */
  
  cgi_quit();
  return(0);
}


/* Create an "<a href>" tag: */

void ahref(int turn, int x, int y)
{
  int xx, yy;
  
  printf("<a href=\"tictactoe.cgi?turn=%d&x=%d&y=%d&board=",
	 turn, x, y);
  
  for (yy = 0; yy < 3; yy++)
    {
      for (xx = 0; xx < 3; xx++)
	{
	  printf("%d", board[yy][xx]);
	}
    }
  
  printf("\">");
}
